/*
 * @Author       : ChenJie
 * @Date         : 2022-02-10 11:43:56
 * @Version      : V3.0
 * @LastEditors  : ChenJie
 * @LastEditTime : 2022-02-14 15:50:00
 * @Description  : file content
 * @FilePath     : \S32K146_4G\src\AppTaskUart1.c
 */
/*
 * AppTaskUart1.c
 *  wifi的串口函数
 *  Created on: 2022年5月11日
 *      Author: cw
 */
#include "AppTaskUart1.h"
#include "stdio.h"
/***************************************************
esp8266使用的串口1
esp8266默认TCP默认断开时间为180s

主机配置
ESP8266作服务器时，IP固定为192.168.4.1
AT+CWMODE=2 								//设置模块 WIFI 模式为 AP 模式
AT+CWSAP_DEF=”my_test1”,”12345678”,1,4 		//设置模块的 AP 参数：SSID 为my_test1，密码为12345678，通道号为 1，加密方式为：WPA_WPA2_PSK
AT+CIPMUX=1 								//开启多连接
AT+CIPSERVER=1,8086 						//开启 SERVER 模式，设置端口为 8086
AT+CIPSEND=0,25 							//向 ID0 发送 25 字节数据包

从机配置
AT+CWMODE=1 								//设置模块 WIFI 模式为 STA 模式
AT+CWJAP_DEF="my_test1","12345678" 			//加入 WIFI 热点
AT+CIPMUX=0 								//开启单连接
AT+CIPSTART="TCP","192.168.4.1",8086		//建立 TCP 连接到” 192.168.4.1”,8086
AT+CIPMODE=1 								//开启透传模式
AT+CIPSEND 									//开始传输
****************************************************/
typedef sint8 (*pFunc)(char *PSendStr, char *pReadStr, uint8 CmdIdx, uint16 pReadLen);

typedef struct
{
	ATCmd cmd; /*指令序号*/
    char *str; /*指令内容*/
    pFunc cb;  /*对应的执行*/
} ATCmdFunc_t;
static void wifi_init(void);
static void tcp_send_data(void);
static void AtcmdTransmit(sint8 CmdIdx, uint8 *SetValuePtr, uint16 SetValueLen, sint8 *retFunc);
static sint8 at_callbackFunc(char *PSendStr, char *pReadStr, uint8 CmdIdx, uint16 pReadLen);
static void wifiDataDecode(uint8 ConnId,uint8 *dataIn,uint16 dataLen);
volatile uint32 TimerCounter = 0; //ms单位
sint8 tcpSendFunc(uint8 TcpConnectId, uint8 *SendDataPtr, uint16 SendDataLen);

typedef enum
{
	TCP_IDLE = 0,
	TCP_OPEN_WIFI,
	TCP_SERVER_OK,
	TCP_SERVER_CLOSE,
}TCP_SERVER;

TCP_SERVER tcp_server_process_status;


const ATCmdFunc_t Atcmdfunc[] =
{
	{AT_CMD_TEST, "AT\r\n", at_callbackFunc},
	{AT_SET_MODE, "AT+CWMODE=2\r\n", at_callbackFunc},  /* 建立SoftAP 模式 */
	{AT_OPEN_MASTER_WIFI, "AT+CWSAP_DEF=\"Hoisting\",\"jx999999\",1,4\r\n", at_callbackFunc},//开启WIFI
	{AT_SET_MUX, "AT+CIPMUX=1\r\n", at_callbackFunc},  /* 设置多链接模式 */
	{AT_OPEN_SERVER, "AT+CIPSERVER=1,5588\r\n", at_callbackFunc}, /* 打开服务器端口 */
	{AT_CIPAP_DEF, "AT+CIPAP_DEF=\"192.168.3.177\",\"192.168.3.177\",\"255.255.255.0\"\r\n", at_callbackFunc}, /* 打开服务器端口 */
	{AT_CIFSR, "AT+CIFSR\r\n", at_callbackFunc},        /* 查看IP信息 */
	{AT_SEND_DATA, "AT+CIPSEND=0,", at_callbackFunc},
	{AT_TCP_CLOSE, "AT+CIPCLOSE=5\r\n", at_callbackFunc},
	{AT_CHECK_LINK_STATUS, "AT+CIPSTATUS\r\n", at_callbackFunc},
};


void wifi_task(void *pvParameters)
{
	(void)pvParameters;
	sint8 ATRet = -1;
	uint16 ReadLen = 0;
	uint8 AtCmdRead[150] = {0};
	uint8 UartDataBuffer[512] = {0};
	//TimerHandle_t softTimer1 = xTimerCreate("softTimer1", 10, pdTRUE, (void *)0, vTimerCallback);
    Dio_WriteChannel(DioConf_DioChannel_PTD16_GPIO_OUT_MCU_WIFI_POW_EN, STD_ON);//wifi开机
	vTaskDelay(pdMS_TO_TICKS(5000));//开机后等一段时间模块可能会返回开机后的指令 不做处理
	wifi_init();
	tcp_server_process_status = TCP_IDLE;
	uint16 pReadLen = 0;
	while(1)
	{
		switch(tcp_server_process_status)
		{

			case TCP_IDLE://等待从机加入wifi，等待从机连接服务器
			{

				memset(UartDataBuffer,0x00,sizeof(UartDataBuffer));
				UART_Receive_Data(UART_LPUART1,UartDataBuffer,&pReadLen,100);
				if(pReadLen>0)
				{
					if(strstr((char *)UartDataBuffer,(char *)("+STA_CONNECTED:")))//从机连上wifi
					{
						tcp_server_process_status = TCP_IDLE;
					}
					else if(strstr((char *)UartDataBuffer,(char *)("+STA_DISCONNECTED:"))||strstr((char *)UartDataBuffer,(char *)("CLOSED")))//从机断开wifi
					{
						tcp_server_process_status = TCP_SERVER_CLOSE;
					}
					else if(strstr((char *)UartDataBuffer,(char *)("CONNECT"))) //从机tcp连上服务器
					{
						tcp_server_process_status = TCP_SERVER_OK;
						wifiConnFlg = true;
					}
				}

				break;
			}
			case TCP_SERVER_OK://可以正常进行数据交互
			{
				/*********主机收到上位机的指令,向WIFI从机发送数据*********/
				if(wifiAskFlg)
				{
					sint8 outValue = -1;
					uint8 McuDataSend[10] = {1,1,1,1,1,1,1,1,1,1};
					sint8 QueueRet = xQueueReceive(McuDataSend_wifi,McuDataSend,0);
					uint8 DataSendLen = sizeof(McuDataSend);
					char AtCmdSend[10] = {0};
					uint8 AtCmdSendTotalLen = 0;
					sprintf(AtCmdSend, "%d\r\n",DataSendLen);
					AtCmdSendTotalLen = strlen(AtCmdSend);
					AtcmdTransmit(AT_SEND_DATA, AtCmdSend, AtCmdSendTotalLen, &ATRet);
					if (ATRet == 0)
					{
						vTaskDelay(pdMS_TO_TICKS(100));
						memset(UartDataBuffer, 0x00, sizeof(UartDataBuffer));
						UART_Query_Data(UART_LPUART1, UART_LPUART1, McuDataSend, DataSendLen, UartDataBuffer, &ReadLen, 1000);
						if ((uint8 *)strstr((char *)UartDataBuffer, (char *)("SEND OK")))
						{
							outValue = 0;
						}
					}
					wifiAskFlg = false;
				}

				/*********************主机收到WIFI从机的数据****************/
				memset(UartDataBuffer, 0x00, sizeof(UartDataBuffer));
				UART_Receive_Data(UART_LPUART1,UartDataBuffer,&ReadLen,100);
				if(ReadLen>0)
				{
					if(strstr((char *)UartDataBuffer,(char *)("+STA_DISCONNECTED")))//从机tcp断开
					{
						tcp_server_process_status = TCP_SERVER_CLOSE;
						wifiAskFlg = false;
					}
					else if(strstr((char *)UartDataBuffer,(char *)("CLOSED")))//从机tcp断开与服务器的链接
					{
						tcp_server_process_status = TCP_SERVER_CLOSE;
						wifiAskFlg = false;
					}
					else if(strstr((char *)UartDataBuffer,(char *)("+IPD")))//收到从机的数据
					{
						uint8 *retptr = NULL;
						uint16 TcpDataLen = 0;
						uint16 NumCalTemp = 1;
						retptr = (uint8 *)strstr((char *)UartDataBuffer, (char *)(":"));
						for (uint8 i = 0; i < 3; i++)
						{
							if (*(retptr - i - 1) == ',')
							{
								break;
							}
							TcpDataLen = TcpDataLen + CharToHex(*(retptr - i - 1)) * NumCalTemp;
							NumCalTemp = NumCalTemp * 10;
						}
						tcpSendFunc(0,retptr+1,TcpDataLen);

						wifiDataDecode(0,retptr+1,TcpDataLen);
						QueueDataTrans_Type DataSend={0, NULL};
						DataSend.RealLen = TcpDataLen;
						DataSend.DataPtr = malloc(TcpDataLen+1);
						memset(DataSend.DataPtr,0x00,ReadLen+1);
						memcpy(DataSend.DataPtr,retptr+1,DataSend.RealLen);
						sint8 McuDataRet = -1;
						McuDataRet= xQueueSend(McuDataRecv_wifi,&DataSend,1000);
						if(McuDataRet != pdPASS)
						{
							if(DataSend.DataPtr != NULL)
							{
								free(DataSend.DataPtr);
							}
							DataSend.DataPtr = NULL;
						}
					}
				}
				break;
			}
			case TCP_SERVER_CLOSE://关闭所有链接
			{
				tcp_server_process_status = TCP_IDLE;
				break;
			}
		}
	}
}
/************************************************************************/
static void wifiDataDecode(uint8 ConnId,uint8 *dataIn,uint16 dataLen)
{
	uint8 lockState = 0;
	uint8 accState = 0;
	uint8 lockDrvState = 0;
	uint8 procState = 0;
	uint8 cmd = 0;
	uint8 timeArray[8];
	uint8 dataHeader[3]={0x68,0x23,0x24};
	uint8 dataVinInfo[17]={0};
	uint8 dataInLen = 0;
	uint8 dataOutLen = 0;
	uint8 dataOutBuffer[128];
	dataInLen = (*(dataIn+4)<<8)|(*(dataIn+3));
	memcpy(dataVinInfo,dataIn+5,17);
	if(dataLen==dataInLen)
	{
		cmd = *(dataIn+5);
		memcpy(timeArray,dataIn+23,8);
		switch(cmd)
		{
			case 0x03://heartData
			{
				lockState = *(dataIn+31);
				accState = *(dataIn+32);
				lockDrvState = *(dataIn+32);
				memcpy(dataOutBuffer + dataOutLen,dataHeader,3);
				dataOutLen = dataOutLen + 3;
				dataOutLen = dataOutLen + 2;//长度跳过
				memcpy(dataOutBuffer + dataOutLen,dataVinInfo,17);
				dataOutLen = dataOutLen + 17;
				memcpy(dataOutBuffer + dataOutLen,timeArray,8);
				dataOutLen = dataOutLen + 8;//未完成
				tcpSendFunc(ConnId,dataOutBuffer,dataOutLen);
				break;
			}
			default:
				break;
		}
	}
}


static void tcp_send_data(void)
{
	/*组包数据*/
	uint16 DataSendLen = 0;
	uint8 UartRecvData[256];
	uint8 *pSendData = NULL;
	sint8 ATRet = -1;
	uint8 ReadLen = 0;
	sint8 outValue = -1;
	/****************************************/
	uint8 send_buf[]="it's from esp8266 server...";
	DataSendLen = strlen(send_buf);
	char AtCmdSend[10] = {0};
	uint8 AtCmdSendTotalLen = 0;
	sprintf(AtCmdSend, "%d\r\n",DataSendLen); //此处需要优化
	AtCmdSendTotalLen = strlen(AtCmdSend);
	AtcmdTransmit(AT_SEND_DATA, AtCmdSend, AtCmdSendTotalLen, &ATRet);
	if (ATRet == 0)
	{
		UART_Query_Data(UART_LPUART1, UART_LPUART1, send_buf, DataSendLen, UartRecvData, &ReadLen, 1000);
		if ((uint8 *)strstr((char *)UartRecvData, (char *)("SEND OK")))
		{
			outValue = 0;
		}
	}
	if(pSendData!=NULL)
	{
		vPortFree(pSendData);
	}
	pSendData = NULL;
}
sint8 tcpSendFunc(uint8 TcpConnectId, uint8 *SendDataPtr, uint16 SendDataLen)
{
	sint8 outValue = -1;
	uint8  sendErrConuter= 0;
	uint16 ReadLen = 0;
	char AtCmdSend[30] = {0};
	uint8 AtCmdSendTotalLen = 0;
	uint8 UartRecvPtr[128];
	uint8 ret = 0;
	sprintf(AtCmdSend, "AT+CIPSEND=%d,%d\r\n", TcpConnectId, SendDataLen);
	AtCmdSendTotalLen = mstrlen(AtCmdSend);
	while(outValue!=0&&sendErrConuter<3)
	{
		ret = UART_Query_Data(UART_LPUART1, UART_LPUART1, (uint8 *)AtCmdSend, AtCmdSendTotalLen, UartRecvPtr, &ReadLen, 1000);
		if (((ret==0)&&(ReadLen>0)&&((uint8 *)strstr((char *)UartRecvPtr, (char *)(">"))))||1)//此IF条件默认通过
		{
			UART_Send_Data(UART_LPUART1, (uint8 *)SendDataPtr, SendDataLen, 100);
			sint8 ret = AtcmdDelayRecvFunc(UART_LPUART1,(char *)("SEND OK"),2000);
			if (ret==0)
			{
				outValue = 0;
			}
			else
			{
				outValue = -2;
				sendErrConuter++;
			}
		}
		else
		{
			outValue = -1;
			sendErrConuter++;
		}
	}
	return outValue;
}
/**********************************************************************************************/
static void wifi_init(void)
{
	//WIFI模块初始化
	uint8 InitStep = 0;
	sint8 ATRet = -1;
	while(1)
	{
		switch (InitStep)
		{
			case 0: // AT指令同步
			{
				AtcmdTransmit(AT_CMD_TEST, NULL, 0, &ATRet);
				if (ATRet == 0)
				{
					InitStep++;
				}
				else
				{
					InitStep = 0;
				}
				break;
			}
			case 1: //设置工作模式
			{
				AtcmdTransmit(AT_SET_MODE, NULL, 0, &ATRet);
				if (ATRet == 0)
				{
					InitStep++;
				}
				else
				{
					InitStep = 0;
				}
				break;
			}
			case 2://开启主机WIFI
			AtcmdTransmit(AT_OPEN_MASTER_WIFI, NULL, 0, &ATRet);
			{
				if(ATRet==0)
				{
					InitStep++;
				}
				else
				{
					InitStep = 0;
				}
			}
			case 3://设置多连接，
			{
				AtcmdTransmit(AT_SET_MUX, NULL, 0, &ATRet);//AT+CIPMUX=1 时才能开启服务器
				if(ATRet==0)
				{
					InitStep++;
				}
				else
				{
					InitStep = 0;
				}
				break;
			}
			case 4://开启服务
			{
				AtcmdTransmit(AT_CIPAP_DEF, NULL, 0, &ATRet);
				{
					if(ATRet==0)
					{
						InitStep++;
					}
					else
					{
						InitStep = 0;
					}
				}
				break;
			}
			case 5:
			{
				AtcmdTransmit(AT_OPEN_SERVER, NULL, 0, &ATRet);
				{
					if(ATRet==0)
					{
						InitStep++;
					}
					else
					{
						InitStep = 0;
					}
				}
				break;
			}
			case 6:
			{
				AtcmdTransmit(AT_CIFSR, NULL, 0, &ATRet);
				{
					if(ATRet==0)
					{
						InitStep++;
					}
					else
					{
						InitStep = 0;
					}
				}
				break;
			}
			default:
			{
				return;
				break;
			}
		}
	}
}


/***********************************************************************************************/
static void AtcmdTransmit(sint8 CmdIdx, uint8 *SetValuePtr, uint16 SetValueLen, sint8 *retFunc)
{
	/* 读取长度 */
	uint16 ReadLen = 0;
	uint8 UartRecvBuffer[256];
	uint8 *PtrATCmdSend = NULL;
	uint8 ATCmdFixedLen = 0;
	uint16 ATCmdTotalLen = 0;
	/* 获取指令长度 */
	ATCmdFixedLen = mstrlen(Atcmdfunc[CmdIdx].str);
	/*  */
	ATCmdTotalLen = ATCmdFixedLen + SetValueLen;

	/* 分配指令内存空间 */
	PtrATCmdSend = malloc(ATCmdTotalLen+1);
	memset(PtrATCmdSend, 0x00, ATCmdTotalLen+1);
	memcpy(PtrATCmdSend, Atcmdfunc[CmdIdx].str, ATCmdFixedLen);
	if (SetValuePtr != NULL)
	{
		memcpy(PtrATCmdSend + ATCmdFixedLen, SetValuePtr, SetValueLen);
	}
	UART_Query_Data(UART_LPUART1, UART_LPUART1, PtrATCmdSend, ATCmdTotalLen, UartRecvBuffer, &ReadLen, 1000);
	*retFunc = Atcmdfunc[CmdIdx].cb(PtrATCmdSend, UartRecvBuffer, CmdIdx, ReadLen);
	if (PtrATCmdSend != NULL)
	{
		memset(PtrATCmdSend, 0x00, ATCmdTotalLen+1);
		free(PtrATCmdSend);
	}
	PtrATCmdSend = NULL;
	return;
}


/*修改*/
static sint8 at_callbackFunc(char *PSendStr, char *pReadStr, uint8 CmdIdx, uint16 pReadLen)
{
	sint8 OutValue = -1;
	uint8 *retptr = NULL;
	char *OkAns = "OK";
	retptr = (uint8 *)strstr((char *)pReadStr, OkAns);
	switch (CmdIdx)
	{
		case AT_CMD_TEST:
		case AT_SET_MODE:
		case AT_OPEN_MASTER_WIFI:
		case AT_SET_MUX:
		case AT_OPEN_SERVER:
		case AT_CIPAP_DEF:
		case AT_CIFSR:
		{
			if (retptr)
			{
				OutValue = 0;
			}
			break;
		}
		case AT_TCP_CLOSE:
		{
			if (retptr)
			{
				OutValue = 0;
			}
			else
			{
				retptr = (uint8 *)strstr((char *)pReadStr, (char *)("CLOSED"));
				{
					if(retptr)
					{
						OutValue = 0;
					}
				}
			}
			break;			
		}
		case AT_CHECK_LINK_STATUS:
		{
			if (retptr)
			{
				retptr = (uint8 *)strstr((char *)pReadStr, (char *)("+CIPSTATUS:"));
				{
					if(retptr)
					{
						OutValue = 0;
					}
				}
			}
			break;
		}
		case AT_SEND_DATA:
		{
			retptr = (uint8 *)strstr((char *)pReadStr, (char *)(">"));
			if (retptr)
			{
				OutValue = 0;
			}
			break;
		}
		//WIFI GOT IP
		default:
			break;
	}
	return OutValue;
}
